package id.kakzaki.blue_thermal_printer;

import android.Manifest;
import android.app.Activity;
import android.app.Application;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Base64;
import android.util.Log;
import android.os.AsyncTask;
import android.os.Handler;
import android.os.Looper;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import cn.bingoogolapple.qrcode.zxing.QRCodeEncoder;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.EventChannel.StreamHandler;
import io.flutter.plugin.common.EventChannel.EventSink;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.common.PluginRegistry.RequestPermissionsResultListener;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import com.gprinter.command.LabelCommand;
import com.journeyapps.barcodescanner.BarcodeEncoder;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


import com.blankj.utilcode.util.LogUtils;

public class BlueThermalPrinterPlugin implements FlutterPlugin, ActivityAware, MethodCallHandler, RequestPermissionsResultListener {

    private static final String TAG = "BThermalPrinterPlugin";
    private static final String NAMESPACE = "blue_thermal_printer";
    private static final int REQUEST_COARSE_LOCATION_PERMISSIONS = 1451;
    private static final UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    private static ConnectedThread THREAD = null;
    private BluetoothAdapter mBluetoothAdapter;
    private ExecutorService executorService;

    private Result pendingResult;

    private EventSink readSink;
    private EventSink statusSink;

    private FlutterPluginBinding pluginBinding;
    private ActivityPluginBinding activityBinding;
    private Object initializationLock = new Object();
    private Context context;
    private MethodChannel channel;

    private EventChannel stateChannel;
    private EventChannel readChannel;
    private BluetoothManager mBluetoothManager;

    private Application application;
    private Activity activity;
    private String lastDevice = "";
//    private Handler handler;

    public static void registerWith(Registrar registrar) {
        final BlueThermalPrinterPlugin instance = new BlueThermalPrinterPlugin();
        //registrar.addRequestPermissionsResultListener(instance);
        Activity activity = registrar.activity();
        Application application = null;
        instance.setup(registrar.messenger(), application, activity, registrar, null);

    }

    public BlueThermalPrinterPlugin() {
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        pluginBinding = binding;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        pluginBinding = null;
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activityBinding = binding;
        executorService = Executors.newFixedThreadPool(1);
        setup(
                pluginBinding.getBinaryMessenger(),
                (Application) pluginBinding.getApplicationContext(),
                activityBinding.getActivity(),
                null,
                activityBinding);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        onDetachedFromActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        detach();
    }

    private void setup(
            final BinaryMessenger messenger,
            final Application application,
            final Activity activity,
            final PluginRegistry.Registrar registrar,
            final ActivityPluginBinding activityBinding) {
        synchronized (initializationLock) {
            Log.i(TAG, "setup");
            this.activity = activity;
            this.application = application;
            this.context = application;
            channel = new MethodChannel(messenger, NAMESPACE + "/methods");
            channel.setMethodCallHandler(this);
            stateChannel = new EventChannel(messenger, NAMESPACE + "/state");
            stateChannel.setStreamHandler(stateStreamHandler);
            readChannel = new EventChannel(messenger, NAMESPACE + "/read");
            readChannel.setStreamHandler(readResultsHandler);
            mBluetoothManager = (BluetoothManager) application.getSystemService(Context.BLUETOOTH_SERVICE);
            mBluetoothAdapter = mBluetoothManager.getAdapter();
            if (registrar != null) {
                // V1 embedding setup for activity listeners.
                registrar.addRequestPermissionsResultListener(this);
            } else {
                // V2 embedding setup for activity listeners.
                activityBinding.addRequestPermissionsResultListener(this);
            }

//            handler = new Handler(Looper.getMainLooper());
        }
    }


    private void detach() {
        Log.i(TAG, "detach");
        context = null;
        activityBinding.removeRequestPermissionsResultListener(this);
        activityBinding = null;
        channel.setMethodCallHandler(null);
        channel = null;
        stateChannel.setStreamHandler(null);
        stateChannel = null;
        mBluetoothAdapter = null;
        mBluetoothManager = null;
        application = null;
    }

    // MethodChannel.Result wrapper that responds on the platform thread.
    private static class MethodResultWrapper implements Result {
        private Result methodResult;
        private Handler handler;

        MethodResultWrapper(Result result) {
            methodResult = result;
            handler = new Handler(Looper.getMainLooper());
        }

        @Override
        public void success(final Object result) {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    methodResult.success(result);
                }
            });
        }

        @Override
        public void error(final String errorCode, final String errorMessage, final Object errorDetails) {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    methodResult.error(errorCode, errorMessage, errorDetails);
                }
            });
        }

        @Override
        public void notImplemented() {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    methodResult.notImplemented();
                }
            });
        }
    }

    @Override
    public void onMethodCall(MethodCall call, Result rawResult) {
        Result result = new MethodResultWrapper(rawResult);

        if (mBluetoothAdapter == null && !"isAvailable".equals(call.method)) {
            result.error("bluetooth_unavailable", "the device does not have bluetooth", null);
            return;
        }

        final Map<String, Object> arguments = call.arguments();

        switch (call.method) {

            case "isAvailable":
                result.success(mBluetoothAdapter != null);
                break;

            case "isOn":
                try {
                    assert mBluetoothAdapter != null;
                    result.success(mBluetoothAdapter.isEnabled());
                } catch (Exception ex) {
                    result.error("Error", ex.getMessage(), exceptionToString(ex));
                }
                break;

            case "isConnected":
                result.success(THREAD != null);
                break;

            case "getLastDevice":
                result.success(lastDevice);
                break;

            case "openSettings":
                ContextCompat.startActivity(context, new Intent(android.provider.Settings.ACTION_BLUETOOTH_SETTINGS),
                        null);
                result.success(true);
                break;

            case "getBondedDevices":
                try {

                    if (ContextCompat.checkSelfPermission(activity,
                            Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {

                        ActivityCompat.requestPermissions(activity,
                                new String[]{Manifest.permission.ACCESS_COARSE_LOCATION}, REQUEST_COARSE_LOCATION_PERMISSIONS);

                        pendingResult = result;
                        break;
                    }

                    getBondedDevices(result);

                } catch (Exception ex) {
                    result.error("Error", ex.getMessage(), exceptionToString(ex));
                }

                break;

            case "connect":
                if (arguments.containsKey("address")) {
                    String address = (String) arguments.get("address");
                    String name = (String) arguments.get("name");
                    Log.d(TAG, "连接蓝牙名字：" + name);
                    lastDevice = name;
                    connect(result, address, name);
                } else {
                    result.error("invalid_argument", "argument 'address' not found", null);
                }
                break;

            case "disconnect":
                disconnect(result);
                break;
            case "writeBytes":
                if (arguments.containsKey("message")) {
                    byte[] message = (byte[]) arguments.get("message");
                    writeBytes(result, message);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            case "printTSCImage":
                if (arguments.containsKey("config") && arguments.containsKey("data")) {
                    final Map<String, Object> config = (Map<String, Object>) arguments.get("config");
                    final List<Map<String, Object>> list = (List<Map<String, Object>>) arguments.get("data");
                    printTSCImage(result, config, list);
                }
                break;
            case "printNewLine":
                printNewLine(result);
                break;

            case "paperCut":
                paperCut(result);
                break;

            case "printImage":
                if (arguments.containsKey("pathImage")) {
                    String pathImage = (String) arguments.get("pathImage");
                    printImage(result, pathImage);
                } else {
                    result.error("invalid_argument", "argument 'pathImage' not found", null);
                }
                break;

            case "printImageBytes":
                if (arguments.containsKey("bytes")) {
                    byte[] bytes = (byte[]) arguments.get("bytes");
                    printImageBytes(result, bytes);
                } else {
                    result.error("invalid_argument", "argument 'bytes' not found", null);
                }
                break;

            case "printQRcode":
                if (arguments.containsKey("textToQR")) {
                    String textToQR = (String) arguments.get("textToQR");
                    int width = (int) arguments.get("width");
                    int height = (int) arguments.get("height");
                    int align = (int) arguments.get("align");
                    int light = (int) arguments.get("light");
                    int weight = (int) arguments.get("weight");
                    if (light == 0 && weight == 0) {
                        printQRcode(result, textToQR, width, height, align);
                    } else {
                        printQRcode(result, textToQR, width, height, light, weight);
                    }
                } else {
                    result.error("invalid_argument", "argument 'textToQR' not found", null);
                }
                break;
            case "printBarCode":
                if (arguments.containsKey("textToBar")) {
                    String textToQR = (String) arguments.get("textToBar");
                    int width = (int) arguments.get("width");
                    int height = (int) arguments.get("height");
                    int align = (int) arguments.get("align");
                    int light = (int) arguments.get("light");
                    int weight = (int) arguments.get("weight");
                    if (light == 0 && weight == 0) {
                        printBarCode(result, textToQR, width, height, align);
                    } else {
                        printBarCode(result, textToQR, width, height, light, weight);
                    }
                } else {
                    result.error("invalid_argument", "argument 'textToQR' not found", null);
                }
                break;
            case "printLeftRight":
                if (arguments.containsKey("string1")) {
                    String string1 = (String) arguments.get("string1");
                    String string2 = (String) arguments.get("string2");
                    int size = (int) arguments.get("size");
                    String charset = (String) arguments.get("charset");
                    String format = (String) arguments.get("format");
                    printLeftRight(result, string1, string2, size, charset, format);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            case "print3Column":
                if (arguments.containsKey("string1")) {
                    String string1 = (String) arguments.get("string1");
                    String string2 = (String) arguments.get("string2");
                    String string3 = (String) arguments.get("string3");
                    int size = (int) arguments.get("size");
                    String charset = (String) arguments.get("charset");
                    String format = (String) arguments.get("format");
                    print3Column(result, string1, string2, string3, size, charset, format);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            case "print4Column":
                if (arguments.containsKey("string1")) {
                    String string1 = (String) arguments.get("string1");
                    String string2 = (String) arguments.get("string2");
                    String string3 = (String) arguments.get("string3");
                    String string4 = (String) arguments.get("string4");
                    int size = (int) arguments.get("size");
                    String charset = (String) arguments.get("charset");
                    String format = (String) arguments.get("format");
                    print4Column(result, string1, string2, string3, string4, size, charset, format);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            case "initPrinter":
                initPrinter(result);
                break;
            case "printMarginLeft":
                if (arguments.containsKey("left")) {
                    int left = (int) arguments.get("left");
                    printMarginLeft(result, left);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }

                break;
            case "getQRcode":
                if (arguments.containsKey("textToQR")) {
                    String textToQR = (String) arguments.get("textToQR");
                    int width = (int) arguments.get("width");
                    int height = (int) arguments.get("height");
                    int align = (int) arguments.get("align");
                    int light = (int) arguments.get("light");
                    int weight = (int) arguments.get("weight");
                    try {
                        getQRcode(result, textToQR, width, height, align);
                    } catch (Exception ex) {
                        result.error("Error", ex.getMessage(), exceptionToString(ex));
                    }
                }
                break;
            case "getBarCode":
                if (arguments.containsKey("textToBar")) {
                    String textToBar = (String) arguments.get("textToBar");
                    int width = (int) arguments.get("width");
                    int height = (int) arguments.get("height");
                    int align = (int) arguments.get("align");
                    int light = (int) arguments.get("light");
                    int weight = (int) arguments.get("weight");
                    try {
                        getBarCode(result, textToBar, width, height, align);
                    } catch (Exception ex) {
                        result.error("Error", ex.getMessage(), exceptionToString(ex));
                    }
                }
                break;
            case "write":
                if (arguments.containsKey("message")) {
                    String message = (String) arguments.get("message");
                    write(result, message);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;


            case "printText":
                if (arguments.containsKey("message")) {
                    String message = (String) arguments.get("message");
                    int size = (int) arguments.get("size");
                    int align = (int) arguments.get("align");
                    boolean bold = (boolean) arguments.get("bold");
                    String charset = (String) arguments.get("charset");
                    String format = (String) arguments.get("format");
                    int light = (int) arguments.get("light");
                    int weight = (int) arguments.get("weight");
                    if (light == 0 && weight == 0) {
                        printText(result, message, size, bold, align, charset, format);
                    } else {
                        printText(result, message, size, bold, light, weight, charset, format);
                    }

                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            default:
                result.notImplemented();
                break;

        }
    }

    /**
     * @param requestCode  requestCode
     * @param permissions  permissions
     * @param grantResults grantResults
     * @return boolean
     */
    @Override
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions,
                                              int[] grantResults) {

        if (requestCode == REQUEST_COARSE_LOCATION_PERMISSIONS) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                getBondedDevices(pendingResult);
            } else {
                pendingResult.error("no_permissions", "this plugin requires location permissions for scanning", null);
                pendingResult = null;
            }
            return true;
        }
        return false;
    }

    private void initPrinter(Result result) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }

        try {
            THREAD.write(PrinterCommands.initPrinter());
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printMarginLeft(Result result, int left) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }

        try {
            THREAD.write(PrinterCommands.printMarginLeft(left));
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    /**
     * @param result result
     */
    private void getBondedDevices(Result result) {

        List<Map<String, Object>> list = new ArrayList<>();

        for (BluetoothDevice device : mBluetoothAdapter.getBondedDevices()) {
            Map<String, Object> ret = new HashMap<>();
            ret.put("address", device.getAddress());
            ret.put("name", device.getName());
            ret.put("type", device.getType());
            int majorDevice = device.getBluetoothClass().getMajorDeviceClass();
            if (majorDevice == 1536) {
                list.add(ret);
            }
        }

        result.success(list);
    }

    private String exceptionToString(Exception ex) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        ex.printStackTrace(pw);
        return sw.toString();
    }

    private void connect(Result result, String address, String name) {

        if (THREAD != null) {
            THREAD.cancel();
            THREAD = null;
            lastDevice = "";
            result.error("connect_error", "already connected", null);
            statusSink.success(0);
            return;
        }

        executorService.submit(() -> {

            try {
                BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
                boolean b = device.fetchUuidsWithSdp();
                LogUtils.i("进来了，device连接方法++++++");
                if (device == null) {
                    if(THREAD != null)
                    THREAD.cancel();
                    THREAD = null;
                    lastDevice = "";
                    result.error("connect_error", "device not found", null);
//                    sendStatus(3);
                    return;
                }

                BluetoothSocket socket = device.createRfcommSocketToServiceRecord(MY_UUID);

                if (socket == null) {
//                    sendStatus(3);
                    if(THREAD != null)
                    THREAD.cancel();
                    THREAD = null;
                    lastDevice = "";
                    result.error("connect_error", "socket connection not established", null);
                    return;
                }

                // Cancel bt discovery, even though we didn't start it
                mBluetoothAdapter.cancelDiscovery();

                try {
                    LogUtils.i("进来了，连接方法++++++");
                    socket.connect();
                    THREAD = new ConnectedThread(socket);
                    THREAD.start();
                    lastDevice = name;
                    result.success(true);
                } catch (IOException ex) {
                    Log.e(TAG, ex.getMessage(), ex);
                    if(THREAD != null)
                    THREAD.cancel();
                    THREAD = null;
                    lastDevice = "";
                    LogUtils.i("异常了，连接方法++++++");
//                    sendStatus(3);
                    result.error("connect_error", ex.getMessage(), exceptionToString(ex));
                }
            } catch (Exception ex) {
                Log.e(TAG, ex.getMessage(), ex);
                if(THREAD != null)
                THREAD.cancel();
                THREAD = null;
                lastDevice = "";
                LogUtils.i("异常了，连接方法++++++");
//                sendStatus(3);
                result.error("connect_error", ex.getMessage(), exceptionToString(ex));
            }


        });


    }

    /**
     * @param result result
     */
    private void disconnect(Result result) {

        if (THREAD == null) {
            result.error("disconnection_error", "not connected", null);
            return;
        }
        executorService.submit(() -> {
            try {
                if(THREAD != null)
                THREAD.cancel();
                THREAD = null;
                lastDevice = "";
                result.success(true);
            } catch (Exception ex) {
                Log.e(TAG, ex.getMessage(), ex);
                result.error("disconnection_error", ex.getMessage(), exceptionToString(ex));
            }
        });
    }

//    private void sendStatus(int event) {
//        handler.post(() -> statusSink.success(event));
//    }

    /**
     * @param result  result
     * @param message message
     */
    private void write(Result result, String message) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }

        try {
            THREAD.write(message.getBytes());
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void writeBytes(Result result, byte[] message) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }

        try {
            result.success(true);
            THREAD.write(message);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printText(Result result, String message, int size, boolean bold,
                           int align, String charset, String format) {
        // Print config "mode"
        byte[] cc = new byte[]{0x1B, 0x21, 0x03}; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[]{0x1B, 0x21, 0x08}; // 1- only bold text
        byte[] bb2 = new byte[]{0x1B, 0x21, 0x20}; // 2- bold with medium text
        byte[] bb3 = new byte[]{0x1B, 0x21, 0x10}; // 3- bold with large text
        byte[] bb4 = new byte[]{0x1B, 0x21, 0x30}; // 4- strong text
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }

        try {
            switch (size) {
                case 0:
                    THREAD.write(cc);
                    break;
                case 1:
                    THREAD.write(bb);
                    break;
                case 2:
                    THREAD.write(bb2);
                    break;
                case 3:
                    THREAD.write(bb3);
                    break;
                case 4:
                    THREAD.write(bb4);
                    break;
            }

            switch (align) {
                case 0:
                    // left align
                    THREAD.write(PrinterCommands.ESC_ALIGN_LEFT);
                    break;
                case 1:
                    // center align
                    THREAD.write(PrinterCommands.ESC_ALIGN_CENTER);
                    break;
                case 2:
                    // right align
                    THREAD.write(PrinterCommands.ESC_ALIGN_RIGHT);
                    break;
            }
            if (format != null) {
                message = String.format(format, message);
            }
            if (charset != null) {
                THREAD.write(message.getBytes(charset));
            } else {
                THREAD.write(message.getBytes());
            }
            THREAD.write(PrinterCommands.FEED_LINE);
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printText(Result result, String message, int size, boolean bold, int light,
                           int weight, String charset, String format) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }

        try {
            THREAD.write(PrinterCommands.fontSizeSetBig(size));
            THREAD.write(PrinterCommands.bold(bold));
            THREAD.write(PrinterCommands.printLocation(light, weight));
            if (format != null) {
                message = String.format(format, message);
            }
            if (charset != null) {
                THREAD.write(message.getBytes(charset));
            } else {
                THREAD.write(message.getBytes());
            }
            THREAD.write(PrinterCommands.FEED_LINE);
            result.success(true);
        } catch (
                Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printLeftRight(Result result, String msg1, String msg2, int size, String
            charset, String format) {
        byte[] cc = new byte[]{0x1B, 0x21, 0x03}; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[]{0x1B, 0x21, 0x08}; // 1- only bold text
        byte[] bb2 = new byte[]{0x1B, 0x21, 0x20}; // 2- bold with medium text
        byte[] bb3 = new byte[]{0x1B, 0x21, 0x10}; // 3- bold with large text
        byte[] bb4 = new byte[]{0x1B, 0x21, 0x30}; // 4- strong text
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            switch (size) {
                case 0:
                    THREAD.write(cc);
                    break;
                case 1:
                    THREAD.write(bb);
                    break;
                case 2:
                    THREAD.write(bb2);
                    break;
                case 3:
                    THREAD.write(bb3);
                    break;
                case 4:
                    THREAD.write(bb4);
                    break;
            }
            THREAD.write(PrinterCommands.ESC_ALIGN_CENTER);
            String line = String.format("%-15s %15s %n", msg1, msg2);
            if (format != null) {
                line = String.format(format, msg1, msg2);
            }
            if (charset != null) {
                THREAD.write(line.getBytes(charset));
            } else {
                THREAD.write(line.getBytes());
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }

    }

    private void print3Column(Result result, String msg1, String msg2, String msg3,
                              int size, String charset, String format) {
        byte[] cc = new byte[]{0x1B, 0x21, 0x03}; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[]{0x1B, 0x21, 0x08}; // 1- only bold text
        byte[] bb2 = new byte[]{0x1B, 0x21, 0x20}; // 2- bold with medium text
        byte[] bb3 = new byte[]{0x1B, 0x21, 0x10}; // 3- bold with large text
        byte[] bb4 = new byte[]{0x1B, 0x21, 0x30}; // 4- strong text
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            switch (size) {
                case 0:
                    THREAD.write(cc);
                    break;
                case 1:
                    THREAD.write(bb);
                    break;
                case 2:
                    THREAD.write(bb2);
                    break;
                case 3:
                    THREAD.write(bb3);
                    break;
                case 4:
                    THREAD.write(bb4);
                    break;
            }
            THREAD.write(PrinterCommands.ESC_ALIGN_CENTER);
            String line = String.format("%-10s %10s %10s %n", msg1, msg2, msg3);
            if (format != null) {
                line = String.format(format, msg1, msg2, msg3);
            }
            if (charset != null) {
                THREAD.write(line.getBytes(charset));
            } else {
                THREAD.write(line.getBytes());
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }

    }

    private void print4Column(Result result, String msg1, String msg2, String msg3, String msg4,
                              int size, String charset, String format) {
        byte[] cc = new byte[]{0x1B, 0x21, 0x03}; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[]{0x1B, 0x21, 0x08}; // 1- only bold text
        byte[] bb2 = new byte[]{0x1B, 0x21, 0x20}; // 2- bold with medium text
        byte[] bb3 = new byte[]{0x1B, 0x21, 0x10}; // 3- bold with large text
        byte[] bb4 = new byte[]{0x1B, 0x21, 0x30}; // 4- strong text
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            switch (size) {
                case 0:
                    THREAD.write(cc);
                    break;
                case 1:
                    THREAD.write(bb);
                    break;
                case 2:
                    THREAD.write(bb2);
                    break;
                case 3:
                    THREAD.write(bb3);
                    break;
                case 4:
                    THREAD.write(bb4);
                    break;
            }
            THREAD.write(PrinterCommands.ESC_ALIGN_CENTER);
            String line = String.format("%-8s %7s %7s %7s %n", msg1, msg2, msg3, msg4);
            if (format != null) {
                line = String.format(format, msg1, msg2, msg3, msg4);
            }
            if (charset != null) {
                THREAD.write(line.getBytes(charset));
            } else {
                THREAD.write(line.getBytes());
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }

    }

    private void printNewLine(Result result) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            THREAD.write(PrinterCommands.FEED_LINE);
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void paperCut(Result result) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            THREAD.write(PrinterCommands.FEED_PAPER_AND_CUT);
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printImage(Result result, String pathImage) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            Bitmap bmp = BitmapFactory.decodeFile(pathImage);
            if (bmp != null) {
                byte[] command = Utils.decodeBitmap(bmp);
                THREAD.write(PrinterCommands.ESC_ALIGN_CENTER);
                THREAD.write(command);
            } else {
                Log.e("Print Photo error", "the file isn't exists");
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printImageBytes(Result result, byte[] bytes) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            Bitmap bmp = BitmapFactory.decodeByteArray(bytes, 0, bytes.length);
            if (bmp != null) {
                byte[] command = Utils.decodeBitmap(bmp);
                THREAD.write(PrinterCommands.ESC_ALIGN_CENTER);
                THREAD.write(command);
            } else {
                Log.e("Print Photo error", "the file isn't exists");
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }


    private void getQRcode(Result result, String textToQR, int width, int height, int align) {
        try {
            byte[] positionBytes = new byte[3];
            MultiFormatWriter multiFormatWriter = new MultiFormatWriter();
            switch (align) {
                case 0:
                    // left align
                    positionBytes = PrinterCommands.ESC_ALIGN_LEFT;
                    break;
                case 1:
                    // center align
                    positionBytes = PrinterCommands.ESC_ALIGN_CENTER;
                    break;
                case 2:
                    // right align
                    positionBytes = PrinterCommands.ESC_ALIGN_RIGHT;
                    break;
            }

            BitMatrix bitMatrix = multiFormatWriter.encode(textToQR, BarcodeFormat.QR_CODE, width, height);
            BarcodeEncoder barcodeEncoder = new BarcodeEncoder();
            Bitmap bmp = barcodeEncoder.createBitmap(bitMatrix);
            byte[] bmpBytes = Utils.decodeBitmap(bmp);
            byte[] bytes = Utils.byteMerger(positionBytes, bmpBytes);
            result.success(bytes);
        } catch (WriterException ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("getQRcode_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printQRcode(Result result, String textToQR, int width, int height, int align) {
        MultiFormatWriter multiFormatWriter = new MultiFormatWriter();
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            switch (align) {
                case 0:
                    // left align
                    THREAD.write(PrinterCommands.ESC_ALIGN_LEFT);
                    break;
                case 1:
                    // center align
                    THREAD.write(PrinterCommands.ESC_ALIGN_CENTER);
                    break;
                case 2:
                    // right align
                    THREAD.write(PrinterCommands.ESC_ALIGN_RIGHT);
                    break;
            }
            BitMatrix bitMatrix = multiFormatWriter.encode(textToQR, BarcodeFormat.QR_CODE, width, height);
            BarcodeEncoder barcodeEncoder = new BarcodeEncoder();
            Bitmap bmp = barcodeEncoder.createBitmap(bitMatrix);

            if (bmp != null) {
                byte[] command = Utils.decodeBitmap(bmp);
                THREAD.write(command);
            } else {
                Log.e("Print Photo error", "the file isn't exists");
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printQRcode(Result result, String textToQR, int width, int height, int light,
                             int weight) {
        MultiFormatWriter multiFormatWriter = new MultiFormatWriter();
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            THREAD.write(PrinterCommands.printLocation(light, weight));
            BitMatrix bitMatrix = multiFormatWriter.encode(textToQR, BarcodeFormat.QR_CODE, width, height);
            BarcodeEncoder barcodeEncoder = new BarcodeEncoder();
            Bitmap bmp = barcodeEncoder.createBitmap(bitMatrix);

            if (bmp != null) {
                byte[] command = Utils.decodeBitmap(bmp);
                THREAD.write(command);
            } else {
                Log.e("Print Photo error", "the file isn't exists");
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void getBarCode(Result result, String textToBar, int width, int height, int align) {
        byte[] positionBytes = new byte[3];
        switch (align) {
            case 0:
                // left align
                positionBytes = PrinterCommands.ESC_ALIGN_LEFT;
                break;
            case 1:
                // center align
                positionBytes = PrinterCommands.ESC_ALIGN_CENTER;
                break;
            case 2:
                // right align
                positionBytes = PrinterCommands.ESC_ALIGN_RIGHT;
                break;
        }
        Bitmap bmp = QRCodeEncoder.syncEncodeBarcode(textToBar, width, height, 0);
        byte[] bmpBytes = Utils.decodeBitmap(bmp);
        byte[] bytes = Utils.byteMerger(positionBytes, bmpBytes);
        result.success(bytes);
    }


    /**
     * 标签打印对象转换
     */
    public void printTSCImage(Result result, Map<String, Object> config, List<Map<String, Object>> list) {
        LabelCommand tsc = new LabelCommand();

        int width = (int) (config.get("width") == null ? 60 : config.get("width")); // 单位：mm
        int height = (int) (config.get("height") == null ? 75 : config.get("height")); // 单位：mm
        int gap = (int) (config.get("gap") == null ? 0 : config.get("gap")); // 单位：mm
        Log.d(TAG, "mapToLabel: width = " + width + ", height = " + height + ", gap = " + gap);
        // 设置标签尺寸宽高，按照实际尺寸设置 单位mm
        tsc.addSize(width, height);
        // 设置标签间隙，按照实际尺寸设置，如果为无间隙纸则设置为0 单位mm
        tsc.addGap(gap);
//        // 设置打印方向
//        tsc.addDirection(LabelCommand.DIRECTION.FORWARD, LabelCommand.MIRROR.NORMAL);
//        // 开启带Response的打印，用于连续打印
//        tsc.addQueryPrinterStatus(LabelCommand.RESPONSE_MODE.ON);
//        // 设置原点坐标
//        tsc.addReference(0, 0);
//        //设置浓度
//        tsc.addDensity(LabelCommand.DENSITY.DNESITY4);
//        // 撕纸模式开启
//        tsc.addTear(EscCommand.ENABLE.ON);
        // 清除打印缓冲区
        tsc.addCls();

        // {type:'text|barcode|qrcode|image', content:'', x:0,y:0}
        for (Map<String, Object> m : list) {
            Log.d(TAG, "printTSCImage: "+m);
            String type = (String) m.get("type");
            String content = (String) m.get("content");
            int x = (int) (m.get("x") == null ? 0 : m.get("x")); //dpi: 1mm约为8个点
            int y = (int) (m.get("y") == null ? 0 : m.get("y"));
            int contentWidth = (int) (m.get("width") == null ? 300 : m.get("width"));

            if ("text".equals(type)) {
                // 绘制简体中文
                tsc.addText(x, y, LabelCommand.FONTTYPE.SIMPLIFIED_CHINESE, LabelCommand.ROTATION.ROTATION_0, LabelCommand.FONTMUL.MUL_1, LabelCommand.FONTMUL.MUL_1, content);
                //打印繁体
                //tsc.addUnicodeText(10,32, LabelCommand.FONTTYPE.TRADITIONAL_CHINESE, LabelCommand.ROTATION.ROTATION_0, LabelCommand.FONTMUL.MUL_1, LabelCommand.FONTMUL.MUL_1,"BIG5碼繁體中文字元","BIG5");
                //打印韩文
                //tsc.addUnicodeText(10,60, LabelCommand.FONTTYPE.KOREAN, LabelCommand.ROTATION.ROTATION_0, LabelCommand.FONTMUL.MUL_1, LabelCommand.FONTMUL.MUL_1,"Korean 지아보 하성","EUC_KR");
            } else if ("barcode".equals(type)) {
                tsc.add1DBarcode(x, y, LabelCommand.BARCODETYPE.CODE128, 100, LabelCommand.READABEL.EANBEL, LabelCommand.ROTATION.ROTATION_0, content);
            } else if ("qrcode".equals(type)) {
                tsc.addQRCode(x, y, LabelCommand.EEC.LEVEL_L, 5, LabelCommand.ROTATION.ROTATION_0, content);
            } else if ("image".equals(type)) {
                byte[] bytes = Base64.decode(content, Base64.DEFAULT);
                Bitmap bitmap = BitmapFactory.decodeByteArray(bytes, 0, bytes.length);
                tsc.addBitmap(x, y, LabelCommand.BITMAP_MODE.OVERWRITE, contentWidth, bitmap);
            }
        }

        // 打印标签
        tsc.addPrint(1, 1);
        // 打印标签后 蜂鸣器响
        tsc.addSound(2, 100);
        //开启钱箱
//        tsc.addCashdrwer(LabelCommand.FOOT.F5, 255, 255);
        // 发送数据
        Byte[] objects = tsc.getCommand().toArray(new Byte[0]);
        writeBytes(result, toPrimitives(objects));
    }

    byte[] toPrimitives(Byte[] oBytes) {
        byte[] bytes = new byte[oBytes.length];
        for (int i = 0; i < oBytes.length; i++) {
            bytes[i] = oBytes[i];
        }
        return bytes;
    }


    private void printBarCode(Result result, String textToBar, int width, int height, int align) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            switch (align) {
                case 0:
                    // left align
                    THREAD.write(PrinterCommands.ESC_ALIGN_LEFT);
                    break;
                case 1:
                    // center align
                    THREAD.write(PrinterCommands.ESC_ALIGN_CENTER);
                    break;
                case 2:
                    // right align
                    THREAD.write(PrinterCommands.ESC_ALIGN_RIGHT);
                    break;
            }
            Bitmap bmp = QRCodeEncoder.syncEncodeBarcode(textToBar, width, height, 0);
            if (bmp != null) {
                THREAD.write(Utils.parseBmpToByte(bmp));
            } else {
                Log.e("Print Photo error", "the file isn't exists");
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private void printBarCode(Result result, String textToBar, int width, int height, int light,
                              int weight) {
        if (THREAD == null) {
            result.error("write_error", "not connected", null);
            return;
        }
        try {
            THREAD.write(PrinterCommands.printLocation(light, weight));
            Bitmap bmp = QRCodeEncoder.syncEncodeBarcode(textToBar, width, height, 0);
            if (bmp != null) {
                THREAD.write(Utils.parseBmpToByte(bmp));
            } else {
                Log.e("Print Photo error", "the file isn't exists");
            }
            result.success(true);
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage(), ex);
            result.error("write_error", ex.getMessage(), exceptionToString(ex));
        }
    }

    private class ConnectedThread extends Thread {
        private final BluetoothSocket mmSocket;
        private final InputStream inputStream;
        private final OutputStream outputStream;

        ConnectedThread(BluetoothSocket socket) {
            mmSocket = socket;
            InputStream tmpIn = null;
            OutputStream tmpOut = null;

            try {
                tmpIn = socket.getInputStream();
                tmpOut = socket.getOutputStream();
            } catch (IOException e) {
                e.printStackTrace();
            }
            inputStream = tmpIn;
            outputStream = tmpOut;
        }

        public void run() {
            byte[] buffer = new byte[1024];
            int bytes;
            while (true) {
                try {
                    bytes = inputStream.read(buffer);
                    readSink.success(new String(buffer, 0, bytes));
                } catch (NullPointerException e) {
                    break;
                } catch (IOException e) {
                    break;
                }
            }
        }

        public void write(byte[] bytes) {
            try {
                outputStream.write(bytes);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        public void cancel() {
            try {
                outputStream.flush();
                outputStream.close();

                inputStream.close();

                mmSocket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private final StreamHandler stateStreamHandler = new StreamHandler() {

        private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                final String action = intent.getAction();

                Log.d(TAG, "蓝牙广播：" + action);

                if (BluetoothAdapter.ACTION_STATE_CHANGED.equals(action)) {
                    THREAD = null;
                    statusSink.success(intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, -1));
                } else if (BluetoothDevice.ACTION_ACL_CONNECTED.equals(action)) {
                    statusSink.success(1);
                } else if (BluetoothDevice.ACTION_ACL_DISCONNECTED.equals(action)) {
                    THREAD = null;
                    statusSink.success(0);
                }
            }
        };

        @Override
        public void onListen(Object o, EventSink eventSink) {
            statusSink = eventSink;
            Log.d(TAG, "注册蓝牙广播：");
            context.registerReceiver(mReceiver, new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED));

            context.registerReceiver(mReceiver, new IntentFilter(BluetoothDevice.ACTION_ACL_CONNECTED));

            context.registerReceiver(mReceiver, new IntentFilter(BluetoothDevice.ACTION_ACL_DISCONNECTED));
        }

        @Override
        public void onCancel(Object o) {
            statusSink = null;
            Log.d(TAG, "取消注册蓝牙广播：");
            context.unregisterReceiver(mReceiver);
        }
    };

    private final StreamHandler readResultsHandler = new StreamHandler() {
        @Override
        public void onListen(Object o, EventSink eventSink) {
            Log.d(TAG, "回调蓝牙广播：");
            readSink = eventSink;
        }

        @Override
        public void onCancel(Object o) {
            readSink = null;
        }
    };
}