import 'package:blue_thermal_printer/blue_thermal_printer.dart';

class BlueThermalPrinter {
  static const int STATE_OFF = 10;
  static const int STATE_TURNING_ON = 11;
  static const int STATE_ON = 12;
  static const int STATE_TURNING_OFF = 13;
  static const int STATE_BLE_TURNING_ON = 14;
  static const int STATE_BLE_ON = 15;
  static const int STATE_BLE_TURNING_OFF = 16;
  static const int ERROR = -1;
  static const int CONNECTED = 1;
  static const int DISCONNECTED = 0;

  static const String namespace = 'blue_thermal_printer';

  static const MethodChannel _channel =
      const MethodChannel('$namespace/methods');

  static const EventChannel _readChannel =
      const EventChannel('$namespace/read');

  static const EventChannel _stateChannel =
      const EventChannel('$namespace/state');

  final StreamController<MethodCall> _methodStreamController =
      new StreamController.broadcast();

  static BlueThermalPrinter _instance = new BlueThermalPrinter._();

  static BlueThermalPrinter get instance => _instance;

  BlueThermalPrinter._() {
    _channel.setMethodCallHandler((MethodCall call) async {
      _methodStreamController.add(call);
    });
  }

  ///onStateChanged()
  Stream<int?> onStateChanged() =>
      _stateChannel.receiveBroadcastStream().map((buffer) => buffer);

  ///onRead()
  Stream<String> onRead() =>
      _readChannel.receiveBroadcastStream().map((buffer) => buffer.toString());

  Future<bool?> get isAvailable async =>
      await _channel.invokeMethod('isAvailable');

  Future<bool?> get isOn async => await _channel.invokeMethod('isOn');

  Future<bool?> get isConnected async =>
      await _channel.invokeMethod('isConnected');

  Future<bool?> get openSettings async =>
      await _channel.invokeMethod('openSettings');

  ///getBondedDevices()
  Future<List<BluetoothDevice>> getBondedDevices() async {
    final List list = await (_channel.invokeMethod('getBondedDevices'));
    return list.map((map) => BluetoothDevice.fromMap(map)).toList();
  }

  Future<String> getLastDevice() async {
    final String getLastDeviceName =
        await (_channel.invokeMethod('getLastDevice'));
    return getLastDeviceName;
  }

  ///printQRcode(String textToQR, int width, int height, int align)

  Future<List<int>> getQRcode(String textToQR, int width, int height, int align,
      {int? light = 0, int? weight = 0}) async {
    final List<int> QRcode = await (_channel.invokeMethod('getQRcode', {
      'textToQR': textToQR,
      'width': width,
      'height': height,
      'align': align,
      'light': light,
      'weight': weight
    }));
    return QRcode;
  }

  Future<List<int>> getBarCode(
      String textToBar, int width, int height, int align,
      {int? light = 0, int? weight = 0}) async {
    final List<int> barCode = await (_channel.invokeMethod('getBarCode', {
      'textToBar': textToBar,
      'width': width,
      'height': height,
      'align': align,
      'light': light,
      'weight': weight
    }));
    return barCode;
  }

  ///connect(BluetoothDevice device)
  Future<dynamic> connect(BluetoothDevice device) =>
      _channel.invokeMethod('connect', device.toMap());

  ///disconnect()
  Future<dynamic> disconnect() => _channel.invokeMethod('disconnect');

  ///write(String message)
  Future<dynamic> write(String message) =>
      _channel.invokeMethod('write', {'message': message});

  ///writeBytes(Uint8List message)
  Future<dynamic> writeBytes(Uint8List message) =>
      _channel.invokeMethod('writeBytes', {'message': message});

  Future<dynamic> printTSCImage(Map<String, dynamic> config, List<Map<String, dynamic>> data) {
    Map<String, Object> args = Map();
    args['config'] = config;
    args['data'] = data;
    print("---- $args");
    _channel.invokeMethod('printTSCImage', args);
    return Future.value(true);
  }


  ///iOS平台使用
  ///printSampleCode(String qrcode, String barcode,
  ///         String title, String subTitle, String smallSubTitle, String bottomContent)
  Future<dynamic> printSampleCode(String qrcode, String barcode, String title,
          String subTitle, String smallSubTitle, String bottomContent) =>
      _channel.invokeMethod('printSampleCode', {
        'qrcode': qrcode,
        'barcode': barcode,
        'title': title,
        'subTitle': subTitle,
        'smallSubTitle': smallSubTitle,
        'bottomContent': bottomContent,
      });

  ///repeatScan()
  Future<dynamic> repeatScan() => _channel.invokeMethod('repeatScan');

  ///printCustom(String message, int size, int align,{String? charset})
  Future<dynamic> printText(String message, int size, int align,
          {String? charset,
          String? format,
          int? light = 0,
          int? weight = 0,
          bool? bold = false}) =>
      _channel.invokeMethod('printText', {
        'message': message,
        'size': size,
        'bold': bold,
        'align': align,
        'charset': charset,
        'format': format,
        'light': light,
        'weight': weight
      });

  ///printNewLine()
  Future<dynamic> printNewLine() => _channel.invokeMethod('printNewLine');

  ///paperCut()
  Future<dynamic> paperCut() => _channel.invokeMethod('paperCut');

  ///initPrinter()
  Future<dynamic> initPrinter() => _channel.invokeMethod('initPrinter');

  ///initMarginLeft(int left)
  Future<dynamic> printMarginLeft(int left) =>
      _channel.invokeMethod('printMarginLeft', {'left': left});

  ///printImage(String pathImage)
  Future<dynamic> printImage(String pathImage) =>
      _channel.invokeMethod('printImage', {'pathImage': pathImage});

  ///printImageBytes(Uint8List bytes)
  Future<dynamic> printImageBytes(Uint8List bytes) =>
      _channel.invokeMethod('printImageBytes', {'bytes': bytes});

  ///printQRcode(String textToQR, int width, int height, int align)
  Future<dynamic> printQRcode(String textToQR, int width, int height, int align,
          {int? light = 0, int? weight = 0}) =>
      _channel.invokeMethod('printQRcode', {
        'textToQR': textToQR,
        'width': width,
        'height': height,
        'align': align,
        'light': light,
        'weight': weight
      });

  ///printQRcode(String textToQR, int width, int height, int align)
  Future<dynamic> printBarCode(
          String textToBar, int width, int height, int align,
          {int? light = 0, int? weight = 0}) =>
      _channel.invokeMethod('printBarCode', {
        'textToBar': textToBar,
        'width': width,
        'height': height,
        'align': align,
        'light': light,
        'weight': weight
      });
}
